// Copyright: Hiroshi Ichikawa
// Lincense: New BSD Lincense

#include "tx_swig.h"

void Builder::add(const std::string& word) {
  word_list_.push_back(word);
}

void Builder::add_all(const std::vector<std::string>& words) {
  word_list_.insert(word_list_.end(), words.begin(), words.end());
}

void Builder::build(const char* file_name) {
  tx_.build(word_list_, file_name);
  word_list_.clear();
}

Index::Index(const char* file_name) {
  tx_.read(file_name);
};

int Index::longest_prefix(
    const char* str, int pos/*= 0*/, int len/*= -1*/, bool match_prefix/*= false*/) {
  validate_str(str, pos, len);
  std::size_t plen;
  if (tx_.searchLongestPrefix(str + pos, len, plen, match_prefix)
      != tx_tool::tx::NOTFOUND) {
    return plen;
  } else {
    return -1;
  }
}

bool Index::include(const char* str, int pos/*= 0*/, int len/*= -1*/) {
  validate_str(str, pos, len);
  return longest_prefix(str, pos, len) == len;
}

std::vector<std::string> Index::search_prefixes(const char* str, int pos/*= 0*/, int len/*= -1*/) {
  validate_str(str, pos, len);
  std::vector<std::string> ret;
  tx_.searchPrefixes(str + pos, len, ret);
  return ret;
}

std::vector<std::string> Index::search_expansions(
    const char* str, int pos/*= 0*/, int len/*= -1*/, int limit/*= 0*/) {
  validate_str(str, pos, len);
  std::vector<std::string> ret;
  tx_.searchExpansions(str + pos, len, ret, limit);
  return ret;
}

void Index::validate_str(const char* str, int& pos, int& len) {
  int max_len= std::strlen(str);
  if (pos > max_len) pos= max_len;
  if (len < 0 || len > max_len - pos) len= max_len - pos;
}
